/*

  Gives info about a sample. Using libsndfile: http://www.zip.com.au/~erikd/libsndfile/

 */

/*                                                                              */
/* This program is free software; you can redistribute it and/or                */
/* modify it under the terms of the GNU General Public License                  */
/* as published by the Free Software Foundation; either version 2               */
/* of the License, or (at your option) any later version.                       */
/*                                                                              */
/* This program is distributed in the hope that it will be useful,              */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of               */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                */
/* GNU General Public License for more details.                                 */
/*                                                                              */
/* You should have received a copy of the GNU General Public License            */
/* along with this program; if not, write to the Free Software                  */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.  */
/*                                                                              */
/* ---------------------------------------------------------------------------- */

#include "m_pd.h"
#include <stdio.h>

#include <sndfile.h>

static char *version = "k_sfinfo v0.1, written by Kjetil S. Matheussen, k.s.matheussen@notam02.no";

 
typedef struct k_sfinfo
{
  t_object x_ob;
  t_inlet  *x_inleft;               /* inlet */

  t_outlet *x_outlet1;               /* result in seconds*/
  t_outlet *x_outlet2;               /* result in frames*/
  t_outlet *x_outlet3;               /* result in samplerate*/
  t_outlet *x_outlet4;               /* number of channels*/
  t_outlet *x_outlet5;               /* number of bits*/
  t_outlet *x_outlet6;               /* format */

  t_float  x_length;
  t_float  x_frames;
  t_float  x_samplerate;
  t_float  x_channels;
  t_float  x_bits;
  t_float  x_format;

} t_k_sfinfo;

static void k_sfinfo_bang(t_k_sfinfo *x)
{
  outlet_float(x->x_outlet1, x->x_length);
  outlet_float(x->x_outlet2, x->x_frames);
  outlet_float(x->x_outlet3, x->x_samplerate);
  outlet_float(x->x_outlet4, x->x_channels);
  outlet_float(x->x_outlet5, x->x_bits);
  outlet_float(x->x_outlet6, x->x_format);
}

static void k_sfinfo_symbol(t_k_sfinfo *x, t_symbol *s)
{
  char *filename=s->s_name;

  SF_INFO sfinfo;
  SNDFILE *infile=sf_open_read(filename,&sfinfo);

  if(infile!=NULL){
    x->x_length = (t_float)sfinfo.samples/sfinfo.samplerate;
    x->x_frames=sfinfo.samples;
    x->x_samplerate=sfinfo.samplerate;
    x->x_channels=sfinfo.channels;
    x->x_bits=sfinfo.pcmbitwidth;
    x->x_format=sfinfo.format;
    sf_close(infile);
  }else{
    x->x_length=0.0f;
    x->x_frames=0.0f;
    x->x_samplerate=0.0f;
    x->x_channels=0.0f;
    x->x_bits=0.0f;
    x->x_format=0.0f;
  }

  //  post("name: -%s-\n",filename);

  k_sfinfo_bang(x);

}

static t_class *k_sfinfo_class;

static void *k_sfinfo_new(t_symbol *s, t_int argc, t_atom* argv)
{

  t_k_sfinfo *x = (t_k_sfinfo *)pd_new(k_sfinfo_class);

  x->x_length=0.0f;
  x->x_frames=0.0f;
  x->x_samplerate=0.0f;
  x->x_channels=0.0f;
  x->x_bits=0.0f;
  x->x_format=0.0f;

  x->x_outlet1 = outlet_new(&x->x_ob, gensym("float"));
  x->x_outlet2 = outlet_new(&x->x_ob, gensym("float"));
  x->x_outlet3 = outlet_new(&x->x_ob, gensym("float"));
  x->x_outlet4 = outlet_new(&x->x_ob, gensym("float"));
  x->x_outlet5 = outlet_new(&x->x_ob, gensym("float"));
  x->x_outlet6 = outlet_new(&x->x_ob, gensym("float"));

  post(version);

  return (void *)x;
}

void k_sfinfo_setup(void)
{
    k_sfinfo_class = class_new(
					  gensym("k_sfinfo"),
					  (t_newmethod)k_sfinfo_new,
					  0, sizeof(t_k_sfinfo), 0, A_GIMME, 0
					  );

    class_addsymbol(k_sfinfo_class, k_sfinfo_symbol);

    class_addbang(k_sfinfo_class, (t_method)k_sfinfo_bang);

    class_sethelpsymbol(k_sfinfo_class, gensym("help-k_sfinfo.pd"));
}

